﻿/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;

namespace Apache.NMS.AMQP.Meta
{
    public class NmsSessionInfo : INmsResource<NmsSessionId>
    {
        public NmsSessionInfo(NmsSessionId sessionId)
        {
            this.Id = sessionId ?? throw new ArgumentNullException(nameof(sessionId), "Session Id object cannot be null");
        }

        public NmsSessionInfo(NmsConnectionInfo connectionInfo, long sessionId)
        {
            if (connectionInfo == null) throw new ArgumentNullException(nameof(connectionInfo), "Connection Info object cannot be null");
            
            this.Id = new NmsSessionId(connectionInfo.Id, sessionId);
        }

        public AcknowledgementMode AcknowledgementMode { get; set; }
        public NmsSessionId Id { get; }
        public bool IsTransacted => AcknowledgementMode == AcknowledgementMode.Transactional;

        protected bool Equals(NmsSessionInfo other)
        {
            return Equals(Id, other.Id);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((NmsSessionInfo) obj);
        }

        public override int GetHashCode()
        {
            return (Id != null ? Id.GetHashCode() : 0);
        }
        
        public override string ToString()
        {
            return $"[{nameof(NmsSessionInfo)}] {nameof(Id)}: {Id}";
        }
    }
}